@@ -1,18 +1,20 @@
-import numpy as np
 import abc
+from typing import Union, Dict
+from nptyping import NDArray
+import numpy as np
+from tqdm.autonotebook import tqdm as progress_bar
 
 from popsynth.utils.meta import Parameter, ParameterMeta
 from popsynth.utils.rejection_sample import rejection_sample
 from popsynth.utils.spherical_geometry import sample_theta_phi
 
-from tqdm.autonotebook import tqdm as progress_bar
-
 
 class DistributionParameter(Parameter):
     pass
 
+
 class Distribution(object, metaclass=ParameterMeta):
-    def __init__(self, name, seed, form):
+    def __init__(self, name: str, seed: int, form: str) -> None:
         """
         A distribution base class
 
@@ -20,31 +22,31 @@ class Distribution(object, metaclass=ParameterMeta):
         :param seed: the random seed
         :param form: the latex form
         :param truth: dictionary holding true parameters
-        :returns: 
-        :rtype: 
+        :returns:
+        :rtype:
 
         """
 
-        self._parameter_storage = {}
-        
-        self._seed = seed
-        self._name = name
-        self._form = form
+        self._parameter_storage = {}  # type: dict
+
+        self._seed = seed  # type: int
+        self._name = name  # type: str
+        self._form = form  # type: str
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self._name
 
     @property
-    def form(self):
+    def form(self) -> str:
         return self._form
 
     @property
-    def params(self):
+    def params(self) -> Dict[str, float]:
         return self._parameter_storage
 
     @property
-    def truth(self):
+    def truth(self) -> Dict[str, float]:
         return self._parameter_storage
 
 
@@ -52,7 +54,7 @@ class SpatialDistribution(Distribution):
 
     r_max = DistributionParameter(vmin=0, default=10)
 
-    def __init__(self, name, seed, form=None):
+    def __init__(self, name: str, seed: int, form: Union[str, None] = None):
         """
         A spatial distribution such as a redshift
         distribution
@@ -61,7 +63,7 @@ class SpatialDistribution(Distribution):
         :param r_max: the maximum distance to sample
         :param seed: the random seed
         :param form: the latex form
-        
+
         """
         self._theta = None
         self._phi = None
@@ -73,9 +75,9 @@ class SpatialDistribution(Distribution):
         """
         the differential volume
 
-        :param distance: 
-        :returns: 
-        :rtype: 
+        :param distance:
+        :returns:
+        :rtype:
 
         """
 
@@ -103,22 +105,22 @@ class SpatialDistribution(Distribution):
         pass
 
     @property
-    def theta(self):
+    def theta(self) -> NDArray[np.float64]:
         return self._theta
 
     @property
-    def phi(self):
+    def phi(self) -> NDArray[np.float64]:
         return self._phi
 
     @property
-    def distances(self):
+    def distances(self) -> NDArray[np.float64]:
         return self._distances
 
-    def draw_sky_positions(self, size):
+    def draw_sky_positions(self, size: int) -> None:
 
         self._theta, self._phi = sample_theta_phi(size)
 
-    def draw_distance(self, size, verbose):
+    def draw_distance(self, size: int, verbose: bool) -> None:
         """
         Draw the distances from the specified dN/dr model
         """
@@ -131,8 +133,10 @@ class SpatialDistribution(Distribution):
         )
 
         # find the maximum point
-        tmp = np.linspace(0.0, self.r_max, 500, dtype=np.float64)
-        ymax = np.max(dNdr(tmp))
+        tmp = np.linspace(
+            0.0, self.r_max, 500, dtype=np.float64
+        )  # type: NDArray[(500,), np.float64]
+        ymax = np.max(dNdr(tmp))  # type: float
 
         # rejection sampling the distribution
         r_out = []
@@ -144,11 +148,11 @@ class SpatialDistribution(Distribution):
 
                     # get am rvs from 0 to the max of the function
 
-                    y = np.random.uniform(low=0, high=ymax)
+                    y = np.random.uniform(low=0, high=ymax)  # type: float
 
                     # get an rvs from 0 to the maximum distance
 
-                    r = np.random.uniform(low=0, high=self.r_max)
+                    r = np.random.uniform(low=0, high=self.r_max)  # type: float
 
                     # compare them
 
@@ -157,15 +161,17 @@ class SpatialDistribution(Distribution):
                         flag = False
         else:
 
-            r_out = rejection_sample(size, ymax, self.r_max, dNdr)
+            r_out = rejection_sample(
+                size, ymax, self.r_max, dNdr
+            )  # type: NDArray[(size,), np.float64]
 
-        self._distances = np.array(r_out)
+        self._distances = np.array(r_out)  # type: NDArray[(size,), np.float64]
 
 
 class LuminosityDistribution(Distribution):
-    def __init__(self, name, seed, form=None):
+    def __init__(self, name: str, seed: int, form: Union[str, None] = None):
         """
-        A luminosity distribution such as a 
+        A luminosity distribution such as a
         distribution
 
         :param name: the name of the distribution
@@ -174,7 +180,9 @@ class LuminosityDistribution(Distribution):
         """
 
         super(LuminosityDistribution, self).__init__(
-            name=name, seed=seed, form=form,
+            name=name,
+            seed=seed,
+            form=form,
         )
 
     @abc.abstractmethod
@@ -182,9 +190,9 @@ class LuminosityDistribution(Distribution):
         """
         The functional form of the distribution
 
-        :param L: 
-        :returns: 
-        :rtype: 
+        :param L:
+        :returns:
+        :rtype:
 
         """
 
