@@ -1,54 +1,65 @@
-import numpy as np
-import scipy.stats as stats
-import scipy.special as sf
-import scipy.integrate as integrate
-import pandas as pd
 import abc
-from IPython.display import display, Math, Markdown
+from typing import Union
 
 import networkx as nx
-
-
-from popsynth.population import Population
-from popsynth.auxiliary_sampler import DerivedLumAuxSampler
-from popsynth.utils.rejection_sample import rejection_sample
-from popsynth.distribution import LuminosityDistribution, SpatialDistribution
-
+import numpy as np
+import pandas as pd
+import scipy.integrate as integrate
+import scipy.special as sf
+import scipy.stats as stats
+from IPython.display import Markdown, Math, display
+from nptyping import NDArray
+from numba import float64, jit, njit, prange
 
 # from popsynth.utils.progress_bar import progress_bar
 from tqdm.autonotebook import tqdm as progress_bar
-from numba import jit, njit, prange, float64
+
+from popsynth.auxiliary_sampler import AuxiliarySampler, DerivedLumAuxSampler
+from popsynth.distribution import LuminosityDistribution, SpatialDistribution
+from popsynth.population import Population
+from popsynth.selection_probability import (
+    BernoulliSelection,
+    HardFluxSelection,
+    SelectionProbabilty,
+    SoftFluxSelection,
+    UnitySelection,
+)
 
 
 class PopulationSynth(object, metaclass=abc.ABCMeta):
-
     def __init__(
         self,
-        spatial_distribution,
-        luminosity_distribution=None,
-        seed=1234,
-        verbose=True,
+        spatial_distribution: SpatialDistribution,
+        luminosity_distribution: Union[LuminosityDistribution, None] = None,
+        seed: int = 1234,
+        verbose: bool = True,
     ):
-        """FIXME! briefly describe function
+        """
+        Basic and generic population synth. One specifies the spatial and luminosity distribution OR
+        derived luminosity distribution and everything is setup.
 
-        :param spatial_distribution: 
-        :param luminosity_distribution: 
-        :param seed: 
-        :returns: 
-        :rtype: 
+        :param spatial_distribution:
+        :param luminosity_distribution:
+        :param seed:
+        :returns:
+        :rtype:
 
         """
 
-        self._n_model = 500
-        self._seed = int(seed)
-        self._model_spaces = {}
-        self._auxiliary_observations = {}
+        self._n_model = 500  # type: int
+        self._seed = int(seed)  # type: int
 
-        self._graph = nx.DiGraph()
+        # this is a container for things passed to
+        # stan
+        self._model_spaces = {}  # type: dict
 
-        self._verbose = verbose
+        self._auxiliary_observations = {}  # type: dict
 
-        self._name = "%s" % spatial_distribution.name
+        self._graph = nx.DiGraph()  # type: nx.Digraph
+
+        self._verbose = verbose  # type: bool
+
+        self._name = "%s" % spatial_distribution.name  # type: str
         if luminosity_distribution is not None:
             self._name = "%s_%s" % (self._name, luminosity_distribution.name)
             assert isinstance(
@@ -59,13 +70,25 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
             spatial_distribution, SpatialDistribution
         ), "the spatial_distribution is the wrong type"
 
-        self._spatial_distribution = spatial_distribution
-        self._luminosity_distribution = luminosity_distribution
+        self._spatial_distribution = spatial_distribution  # type: SpatialDistribution
+        self._luminosity_distribution = (
+            luminosity_distribution
+        )  # type: Union[LuminosityDistribution, None]
 
-        self._has_derived_luminosity = False
-        self._derived_luminosity_sampler = None
+        self._has_derived_luminosity = False  # type: bool
+        self._derived_luminosity_sampler = (
+            None
+        )  # type: Union[DerivedLumAuxSampler, None]
 
-        self._params = {}
+        # set the selections be fully seen unless it is set by the user
+        self._distance_selector = UnitySelection()  # type: SelectionProbabilty
+        self._flux_selector = UnitySelection()  # type: SelectionProbabilty
+
+        # check to see if the selectors are set
+        self._distance_selector_set = False  # type: bool
+        self._flux_selector_set = False  # type: bool
+
+        self._params = {}  # type: dict
 
         # keep a list of parameters here for checking
 
@@ -82,14 +105,12 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
 
         # add the sky sampler
 
-        
-        
     @property
-    def spatial_distribution(self):
+    def spatial_distribution(self) -> SpatialDistribution:
         return self._spatial_distribution
 
     @property
-    def luminosity_distribution(self):
+    def luminosity_distribution(self) -> Union[LuminosityDistribution, None]:
         return self._luminosity_distribution
 
     def add_model_space(self, name, start, stop, log=True):
@@ -111,7 +132,9 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
 
         self._model_spaces[name] = space
 
-    def add_observed_quantity(self, auxiliary_sampler):
+    def add_observed_quantity(
+        self, auxiliary_sampler: Union[DerivedLumAuxSampler, AuxiliarySampler]
+    ):
         """FIXME! briefly describe function
 
         :param auxiliary_sampler:
@@ -132,14 +155,39 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
 
         else:
 
-            assert not auxiliary_sampler.is_secondary, f'{auxiliary_sampler.name} is already set as a secondary sampler!'
-            assert auxiliary_sampler.name not in self._auxiliary_observations, f'{auxiliary_sampler.name} is already registered!'
+            assert (
+                not auxiliary_sampler.is_secondary
+            ), f"{auxiliary_sampler.name} is already set as a secondary sampler!"
+            assert (
+                auxiliary_sampler.name not in self._auxiliary_observations
+            ), f"{auxiliary_sampler.name} is already registered!"
             if self._verbose:
                 print("registering auxilary sampler: %s" % auxiliary_sampler.name)
 
             self._auxiliary_observations[auxiliary_sampler.name] = auxiliary_sampler
 
-    def _prob_det(self, x, boundary, strength):
+    def set_distance_selection(self, selector: SelectionProbabilty) -> None:
+        """
+        Set the selection type for the distance
+        """
+
+        assert isinstance(selector, SelectionProbabilty)
+
+        self._distance_selector = selector
+
+        self._distance_selector_set = True
+
+    def set_flux_selection(self, selector: SelectionProbabilty) -> None:
+        """
+        Set the selection type for the distance
+        """
+        assert isinstance(selector, SelectionProbabilty)
+
+        self._flux_selector = selector
+
+        self._flux_selector_set = True
+
+    def _prob_det(self, x, boundary, strength) -> NDArray[np.float64]:
         """
         Soft detection threshold
 
@@ -151,10 +199,10 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
         return sf.expit(strength * (x - boundary))
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self._name
 
-    def draw_log10_fobs(self, f, f_sigma, size=1):
+    def draw_log10_fobs(self, f, f_sigma, size=1) -> NDArray[np.float64]:
         """
         draw the log10 of the the fluxes
         """
@@ -168,14 +216,14 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
 
     def draw_survey(
         self,
-        boundary,
-        flux_sigma=1.0,
-        strength=10.0,
-        hard_cut=False,
-        distance_probability=None,
-        no_selection=False,
-        verbose=True,
-    ):
+        boundary: float,
+        flux_sigma: float = 1.0,
+        strength: float = 10.0,
+        hard_cut: bool = False,
+        distance_probability: Union[float, None] = None,
+        no_selection: bool = False,
+        verbose: bool = True,
+    ) -> Population:
         """
         Draw the total survey and return a Population object
 
@@ -188,7 +236,7 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
         """
 
         # this stores all the "true" population values from all the samplers
-        truth = dict()
+        truth = dict()  # type: dict
 
         # store the spatial distribution truths
         truth[self._spatial_distribution.name] = self._spatial_distribution.truth
@@ -207,23 +255,26 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
 
         # integrate the population to determine the true number of
         # objects
-        N = integrate.quad(dNdr, 0.0, self._spatial_distribution.r_max)[0]
+        N = integrate.quad(dNdr, 0.0, self._spatial_distribution.r_max)[
+            0
+        ]  # type: float
 
         if verbose:
             print("The volume integral is %f" % N)
 
         # this should be poisson distributed
-        n = np.random.poisson(N)
+        n = np.random.poisson(N)  # type: np.int64
 
-        
         self._spatial_distribution.draw_distance(size=n, verbose=verbose)
 
         # now draw the sky positions
-        
+
         self._spatial_distribution.draw_sky_positions(size=n)
 
-        distances = self._spatial_distribution.distances
-        
+        distances = (
+            self._spatial_distribution.distances
+        )  # type: NDArray[(n,), np.float64]
+
         if verbose:
             print("Expecting %d total objects" % n)
 
@@ -232,12 +283,18 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
 
         #      pbar.update()
 
+        # setup the global selection
+
+        global_selection = UnitySelection()  # type: SelectionProbabilty
+        global_selection.draw(n, verbose=False)
+
         # now we set up the selection that _may_ come
         # from the auxilliary samplers
 
-        auxiliary_selection = np.ones(n, dtype=bool)
+        auxiliary_selection = UnitySelection()  # type: SelectionProbabilty
+        auxiliary_selection.draw(n, verbose=False)
 
-        auxiliary_quantities = {}
+        auxiliary_quantities = {}  # type: dict
 
         # this means the luminosity is not
         # simulated directy
@@ -272,11 +329,13 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
             auxiliary_quantities[self._derived_luminosity_sampler.name] = {
                 "true_values": self._derived_luminosity_sampler.true_values,
                 "obs_values": self._derived_luminosity_sampler.obs_values,
-                "selection": self._derived_luminosity_sampler.selection,
+                "selection": self._derived_luminosity_sampler.selector,
             }
             if verbose:
                 print("Getting luminosity from derived sampler")
-            luminosities = self._derived_luminosity_sampler.compute_luminosity()
+            luminosities = (
+                self._derived_luminosity_sampler.compute_luminosity()
+            )  # type: NDArray[(n,), np.float64]
 
             # collect anything that was sampled here
 
@@ -290,8 +349,8 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
 
                 # first we tell the sampler to go and retrieve all of
                 # its own secondaries
-                
-                properties = v2.get_secondary_properties()
+
+                properties = v2.get_secondary_properties()  # type: dict
 
                 for k3, v3 in properties.items():
 
@@ -309,7 +368,9 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
         else:
             # pbar.update()
             # draw all the values
-            luminosities = self.luminosity_distribution.draw_luminosity(size=n)
+            luminosities = self.luminosity_distribution.draw_luminosity(
+                size=n
+            )  # type: NDArray[(n,), np.float64]
 
             # store the truths from the luminosity distribution
             truth[
@@ -317,7 +378,9 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
             ] = self.luminosity_distribution.truth
 
         # transform the fluxes
-        fluxes = self._spatial_distribution.transform(luminosities, distances)
+        fluxes = self._spatial_distribution.transform(
+            luminosities, distances
+        )  # type: NDArray[(n,), np.float64]
 
         # now sample any auxilary quantities
         # if needed
@@ -353,8 +416,8 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
             auxiliary_quantities[k] = {
                 "true_values": v.true_values,
                 "obs_values": v.obs_values,
-                "selection": v.selection,
-            }
+                "selection": v.selector,
+            }  # type: dict
 
             # collect the secondary values
 
@@ -363,7 +426,7 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
                 # first we tell the sampler to go and retrieve all of
                 # its own secondaries
 
-                properties = v2.get_secondary_properties()
+                properties = v2.get_secondary_properties()  # type: dict
 
                 for k3, v3 in properties.items():
 
@@ -378,164 +441,139 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
 
         # now draw all the observed fluxes
         # this is homoskedastic for now
-        log10_fluxes_obs = self.draw_log10_fobs(fluxes, flux_sigma, size=n)
+        log10_fluxes_obs = self.draw_log10_fobs(
+            fluxes, flux_sigma, size=n
+        )  # type: NDArray[(n,), np.float64]
 
         assert np.alltrue(np.isfinite(log10_fluxes_obs))
-        
+
         # now select them
 
-        if not hard_cut:
+        if not no_selection:
 
-            if verbose:
+            if not self._flux_selector_set:
 
-                print("Applying soft boundary")
+                DeprecationWarning(
+                    "this interface will change and soon you will be required to set the flux selection manually"
+                )
 
-            # compute the detection probability  for the observed values
+                if not hard_cut:
 
-            detection_probability = self._prob_det(
-                log10_fluxes_obs, np.log10(boundary), strength
-            )
+                    self._flux_selector = SoftFluxSelection(boundary, strength)
 
-            selection = []
-            if verbose:
-                for p in progress_bar(
-                    detection_probability, desc="samping detection probability"
-                ):
+                    if verbose:
 
-                    # make a bernoulli draw given the detection probability
+                        print("Applying soft boundary")
 
-                    if stats.bernoulli.rvs(p) == 1:
+                else:
 
-                        selection.append(True)
+                    if verbose:
 
-                    else:
+                        print("Applying hard boundary")
 
-                        selection.append(False)
+                    self._flux_selector = HardFluxSelection(boundary)
+
+            # the hard and soft flux selectors have built in
+            # properties to let us know what type of selection
+            # was made so we can record it
+
+            if self._flux_selector.hard_cut:
+
+                strength = 1.0
 
             else:
 
-                for p in detection_probability:
+                strength = self._flux_selector.strength
 
-                    # make a bernoulli draw given the detection probability
-                    if stats.bernoulli.rvs(p) == 1:
-
-                        selection.append(True)
-
-                    else:
-
-                        selection.append(False)
-
-            selection = np.array(selection)
+            boundary = self._flux_selector.boundary
+            hard_cut = self._flux_selector.hard_cut
 
         else:
 
-            if verbose:
+            # These are just dummies for the no selection case
 
-                print("Applying hard boundary")
+            strength = 1
+            boundary = 1e-99
+            hard_cut = True
 
-            # simply apply a hard cut selection in the data
+        # pass the values the plux selector and draw the selection
+        self._flux_selector.set_observed_flux(10 ** log10_fluxes_obs)
 
-            selection = np.power(10, log10_fluxes_obs) >= boundary
+        self._flux_selector.draw(n)
+
+        #       selection = self._flux_selector.selection
 
         # now apply the selection from the auxilary samplers
 
         for k, v in auxiliary_quantities.items():
 
-            auxiliary_selection = np.logical_and(auxiliary_selection, v["selection"])
+            auxiliary_selection += v["selection"]
 
             if verbose:
 
-                if sum(~v["selection"]) > 0:
+                if v["selection"].n_non_selected > 0:
 
                     print(
                         "Applying selection from %s which selected %d of %d objects"
-                        % (k, sum(v["selection"]), len(v["selection"]))
+                        % (k, v["selection"].n_selected, v["selection"].n_objects)
                     )
 
         if verbose:
 
-            if sum(~auxiliary_selection) > 0:
+            if auxiliary_selection.n_non_selected > 0:
                 print(
                     "Before auxiliary selection there were %d objects selected"
-                    % sum(selection)
+                    % self._flux_selector.n_selected
                 )
 
-        selection = np.logical_and(selection, auxiliary_selection)
+        # now we can add the values onto the global
+        # selection
+        # not in the future we will depreciate the
+        # no selection feature
+        if not no_selection:
 
-        # if we do not want to add a selection effect
-        if no_selection:
-            if self._verbose:
-                print("No Selection! Added back all objects")
+            global_selection += auxiliary_selection
 
-            selection = np.ones_like(selection, dtype=bool)
+            global_selection += self._flux_selector
+
+        # # if we do not want to add a selection effect
+        # if no_selection:
+        #     if self._verbose:
+        #         print("No Selection! Added back all objects")
+
+        #     selection = np.ones_like(
+        #         selection, dtype=bool
+        #     )  # type: NDArray[(n,), np.bool_]
 
         # pbar.update()
-        if sum(selection) == n:
+        if global_selection.n_selected == n:
 
             if verbose:
                 print("NO HIDDEN OBJECTS")
 
-        if (distance_probability is not None) or (distance_probability == 1.0):
-            # pbar.set_description(desc='Selecting sistances')
-            known_distances = []
-            known_distance_idx = []
-            unknown_distance_idx = []
+        if not self._distance_selector_set:
 
-            assert (distance_probability >= 0) and (
-                distance_probability <= 1.0
-            ), "the distance detection must be between 0 and 1"
+            if (distance_probability is not None) or (distance_probability == 1.0):
 
-            if verbose:
-                with progress_bar(
-                    len(distances[selection]), desc="Selecting distances"
-                ) as pbar2:
-                    for i, d in enumerate(distances[selection]):
+                self._distance_selector = BernoulliSelection(distance_probability)
 
-                        # see if we detect the distance
-                        if stats.bernoulli.rvs(distance_probability) == 1:
+        self._distance_selector.draw(size=global_selection.n_selected, verbose=verbose)
 
-                            known_distances.append(d)
-                            known_distance_idx.append(i)
+        known_distances = distances[global_selection.selection][
+            self._distance_selector.selection
+        ]
+        known_distance_idx = self._distance_selector.selection_index
+        unknown_distance_idx = self._distance_selector.non_selection_index
 
-                        else:
-
-                            unknown_distance_idx.append(i)
-
-                        pbar2.update()
-
-            else:
-
-                for i, d in enumerate(distances[selection]):
-
-                    # see if we detect the distance
-                    if stats.bernoulli.rvs(distance_probability) == 1:
-
-                        known_distances.append(d)
-                        known_distance_idx.append(i)
-
-                    else:
-
-                        unknown_distance_idx.append(i)
-
-            if verbose:
-                print("Detected %d distances" % len(known_distances))
-
-        else:
-
-            known_distances = distances[selection]
-            known_distance_idx = [i for i in range(sum(selection))]
-            unknown_distance_idx = []
-        # pbar.update()
-        known_distances = np.array(known_distances)
-        known_distance_idx = np.array(known_distance_idx)
-        unknown_distance_idx = np.array(unknown_distance_idx)
+        if verbose:
+            print("Detected %d distances" % len(known_distances))
 
         if verbose:
             try:
 
                 print(
                     "Deteced %d objects or to a distance of %.2f"
-                    % (sum(selection), max(known_distances))
+                    % (global_selection.n_selected, max(known_distances))
                 )
 
             except:
@@ -561,7 +599,7 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
             unknown_distance_idx=unknown_distance_idx,
             fluxes=fluxes,
             flux_obs=np.power(10, log10_fluxes_obs),
-            selection=selection,
+            selection=global_selection.selection,
             flux_sigma=flux_sigma,
             r_max=self._spatial_distribution.r_max,
             n_model=self._n_model,
@@ -579,11 +617,11 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
             hard_cut=hard_cut,
             distance_probability=distance_probability,
             graph=self.graph,
-            theta = self._spatial_distribution.theta,
-            phi = self._spatial_distribution.phi
+            theta=self._spatial_distribution.theta,
+            phi=self._spatial_distribution.phi,
         )
 
-    def display(self):
+    def display(self) -> None:
         """
         Display the simulation parameters
 
@@ -628,8 +666,8 @@ class PopulationSynth(object, metaclass=abc.ABCMeta):
         """
         builds the graph for all the samplers
 
-        :returns: 
-        :rtype: 
+        :returns:
+        :rtype:
 
         """
 
